

Patch from Stephen Hemminger <shemminger@osdl.org>

Replace brlock in IPV4 with RCU



 25-akpm/net/ipv4/af_inet.c                           |   20 ++++++++++---------
 25-akpm/net/ipv4/ip_output.c                         |    9 +++-----
 25-akpm/net/ipv4/netfilter/ip_conntrack_core.c       |    7 +-----
 25-akpm/net/ipv4/netfilter/ip_conntrack_standalone.c |    4 ---
 25-akpm/net/ipv4/netfilter/ip_nat_core.c             |    1 
 25-akpm/net/ipv4/netfilter/ip_nat_helper.c           |    4 ---
 25-akpm/net/ipv4/netfilter/ip_nat_snmp_basic.c       |    3 --
 25-akpm/net/ipv4/netfilter/ip_nat_standalone.c       |    4 ---
 25-akpm/net/ipv4/netfilter/ip_queue.c                |    4 ---
 25-akpm/net/ipv4/protocol.c                          |   15 +++++---------
 10 files changed, 27 insertions(+), 44 deletions(-)

diff -puN net/ipv4/af_inet.c~brlock-6 net/ipv4/af_inet.c
--- 25/net/ipv4/af_inet.c~brlock-6	Tue Mar 11 16:59:51 2003
+++ 25-akpm/net/ipv4/af_inet.c	Tue Mar 11 16:59:51 2003
@@ -94,7 +94,6 @@
 #include <linux/inet.h>
 #include <linux/igmp.h>
 #include <linux/netdevice.h>
-#include <linux/brlock.h>
 #include <net/ip.h>
 #include <net/protocol.h>
 #include <net/arp.h>
@@ -130,6 +129,7 @@ static kmem_cache_t *raw4_sk_cachep;
  * build a new socket.
  */
 struct list_head inetsw[SOCK_MAX];
+static spinlock_t inetsw_lock = SPIN_LOCK_UNLOCKED;
 
 /* New destruction routine */
 
@@ -337,8 +337,8 @@ static int inet_create(struct socket *so
 
 	/* Look for the requested type/protocol pair. */
 	answer = NULL;
-	br_read_lock_bh(BR_NETPROTO_LOCK);
-	list_for_each(p, &inetsw[sock->type]) {
+	rcu_read_lock();
+	list_for_each_rcu(p, &inetsw[sock->type]) {
 		answer = list_entry(p, struct inet_protosw, list);
 
 		/* Check the non-wild match. */
@@ -356,7 +356,7 @@ static int inet_create(struct socket *so
 		}
 		answer = NULL;
 	}
-	br_read_unlock_bh(BR_NETPROTO_LOCK);
+	rcu_read_unlock();
 
 	err = -ESOCKTNOSUPPORT;
 	if (!answer)
@@ -977,7 +977,7 @@ void inet_register_protosw(struct inet_p
 	int protocol = p->protocol;
 	struct list_head *last_perm;
 
-	br_write_lock_bh(BR_NETPROTO_LOCK);
+	spin_lock_bh(&inetsw_lock);
 
 	if (p->type > SOCK_MAX)
 		goto out_illegal;
@@ -1006,9 +1006,9 @@ void inet_register_protosw(struct inet_p
 	 * non-permanent entry.  This means that when we remove this entry, the 
 	 * system automatically returns to the old behavior.
 	 */
-	list_add(&p->list, last_perm);
+	list_add_rcu(&p->list, last_perm);
 out:
-	br_write_unlock_bh(BR_NETPROTO_LOCK);
+	spin_unlock_bh(&inetsw_lock);
 	return;
 
 out_permanent:
@@ -1030,9 +1030,11 @@ void inet_unregister_protosw(struct inet
 		       "Attempt to unregister permanent protocol %d.\n",
 		       p->protocol);
 	} else {
-		br_write_lock_bh(BR_NETPROTO_LOCK);
+		spin_lock_bh(&inetsw_lock);
 		list_del(&p->list);
-		br_write_unlock_bh(BR_NETPROTO_LOCK);
+		spin_unlock_bh(&inetsw_lock);
+
+		synchronize_kernel();
 	}
 }
 
diff -puN net/ipv4/ip_output.c~brlock-6 net/ipv4/ip_output.c
--- 25/net/ipv4/ip_output.c~brlock-6	Tue Mar 11 16:59:51 2003
+++ 25-akpm/net/ipv4/ip_output.c	Tue Mar 11 16:59:51 2003
@@ -1261,11 +1261,10 @@ void ip_send_reply(struct sock *sk, stru
 
 static struct packet_type ip_packet_type =
 {
-	__constant_htons(ETH_P_IP),
-	NULL,	/* All devices */
-	ip_rcv,
-	(void*)1,
-	NULL,
+	.type = __constant_htons(ETH_P_IP),
+	.dev = NULL,	/* All devices */
+	.func = ip_rcv,
+	.data = (void*)1,
 };
 
 /*
diff -puN net/ipv4/netfilter/ip_conntrack_core.c~brlock-6 net/ipv4/netfilter/ip_conntrack_core.c
--- 25/net/ipv4/netfilter/ip_conntrack_core.c~brlock-6	Tue Mar 11 16:59:51 2003
+++ 25-akpm/net/ipv4/netfilter/ip_conntrack_core.c	Tue Mar 11 16:59:51 2003
@@ -24,7 +24,6 @@
 #include <linux/skbuff.h>
 #include <linux/proc_fs.h>
 #include <linux/vmalloc.h>
-#include <linux/brlock.h>
 #include <net/checksum.h>
 #include <linux/stddef.h>
 #include <linux/sysctl.h>
@@ -1160,8 +1159,7 @@ void ip_conntrack_helper_unregister(stru
 	WRITE_UNLOCK(&ip_conntrack_lock);
 
 	/* Someone could be still looking at the helper in a bh. */
-	br_write_lock_bh(BR_NETPROTO_LOCK);
-	br_write_unlock_bh(BR_NETPROTO_LOCK);
+	synchronize_kernel();
 }
 
 /* Refresh conntrack for this many jiffies. */
@@ -1402,8 +1400,7 @@ void ip_conntrack_cleanup(void)
 	/* This makes sure all current packets have passed through
            netfilter framework.  Roll on, two-stage module
            delete... */
-	br_write_lock_bh(BR_NETPROTO_LOCK);
-	br_write_unlock_bh(BR_NETPROTO_LOCK);
+	synchronize_kernel();
  
  i_see_dead_people:
 	ip_ct_selective_cleanup(kill_all, NULL);
diff -puN net/ipv4/netfilter/ip_conntrack_standalone.c~brlock-6 net/ipv4/netfilter/ip_conntrack_standalone.c
--- 25/net/ipv4/netfilter/ip_conntrack_standalone.c~brlock-6	Tue Mar 11 16:59:51 2003
+++ 25-akpm/net/ipv4/netfilter/ip_conntrack_standalone.c	Tue Mar 11 16:59:51 2003
@@ -15,7 +15,6 @@
 #include <linux/skbuff.h>
 #include <linux/proc_fs.h>
 #include <linux/version.h>
-#include <linux/brlock.h>
 #include <net/checksum.h>
 
 #define ASSERT_READ_LOCK(x) MUST_BE_READ_LOCKED(&ip_conntrack_lock)
@@ -325,8 +324,7 @@ void ip_conntrack_protocol_unregister(st
 	WRITE_UNLOCK(&ip_conntrack_lock);
 	
 	/* Somebody could be still looking at the proto in bh. */
-	br_write_lock_bh(BR_NETPROTO_LOCK);
-	br_write_unlock_bh(BR_NETPROTO_LOCK);
+	synchronize_kernel();
 
 	/* Remove all contrack entries for this protocol */
 	ip_ct_selective_cleanup(kill_proto, &proto->proto);
diff -puN net/ipv4/netfilter/ip_nat_core.c~brlock-6 net/ipv4/netfilter/ip_nat_core.c
--- 25/net/ipv4/netfilter/ip_nat_core.c~brlock-6	Tue Mar 11 16:59:51 2003
+++ 25-akpm/net/ipv4/netfilter/ip_nat_core.c	Tue Mar 11 16:59:51 2003
@@ -8,7 +8,6 @@
 #include <linux/timer.h>
 #include <linux/skbuff.h>
 #include <linux/netfilter_ipv4.h>
-#include <linux/brlock.h>
 #include <linux/vmalloc.h>
 #include <net/checksum.h>
 #include <net/icmp.h>
diff -puN net/ipv4/netfilter/ip_nat_helper.c~brlock-6 net/ipv4/netfilter/ip_nat_helper.c
--- 25/net/ipv4/netfilter/ip_nat_helper.c~brlock-6	Tue Mar 11 16:59:51 2003
+++ 25-akpm/net/ipv4/netfilter/ip_nat_helper.c	Tue Mar 11 16:59:51 2003
@@ -20,7 +20,6 @@
 #include <linux/timer.h>
 #include <linux/skbuff.h>
 #include <linux/netfilter_ipv4.h>
-#include <linux/brlock.h>
 #include <net/checksum.h>
 #include <net/icmp.h>
 #include <net/ip.h>
@@ -545,8 +544,7 @@ void ip_nat_helper_unregister(struct ip_
 	WRITE_UNLOCK(&ip_nat_lock);
 
 	/* Someone could be still looking at the helper in a bh. */
-	br_write_lock_bh(BR_NETPROTO_LOCK);
-	br_write_unlock_bh(BR_NETPROTO_LOCK);
+	synchronize_kernel();
 
 	/* Find anything using it, and umm, kill them.  We can't turn
 	   them into normal connections: if we've adjusted SYNs, then
diff -puN net/ipv4/netfilter/ip_nat_snmp_basic.c~brlock-6 net/ipv4/netfilter/ip_nat_snmp_basic.c
--- 25/net/ipv4/netfilter/ip_nat_snmp_basic.c~brlock-6	Tue Mar 11 16:59:51 2003
+++ 25-akpm/net/ipv4/netfilter/ip_nat_snmp_basic.c	Tue Mar 11 16:59:51 2003
@@ -50,7 +50,6 @@
 #include <linux/netfilter_ipv4.h>
 #include <linux/netfilter_ipv4/ip_nat.h>
 #include <linux/netfilter_ipv4/ip_nat_helper.h>
-#include <linux/brlock.h>
 #include <linux/types.h>
 #include <linux/ip.h>
 #include <net/udp.h>
@@ -1351,8 +1350,6 @@ static void __exit fini(void)
 {
 	ip_nat_helper_unregister(&snmp);
 	ip_nat_helper_unregister(&snmp_trap);
-	br_write_lock_bh(BR_NETPROTO_LOCK);
-	br_write_unlock_bh(BR_NETPROTO_LOCK);
 }
 
 module_init(init);
diff -puN net/ipv4/netfilter/ip_nat_standalone.c~brlock-6 net/ipv4/netfilter/ip_nat_standalone.c
--- 25/net/ipv4/netfilter/ip_nat_standalone.c~brlock-6	Tue Mar 11 16:59:51 2003
+++ 25-akpm/net/ipv4/netfilter/ip_nat_standalone.c	Tue Mar 11 16:59:51 2003
@@ -24,7 +24,6 @@
 #include <net/checksum.h>
 #include <linux/spinlock.h>
 #include <linux/version.h>
-#include <linux/brlock.h>
 
 #define ASSERT_READ_LOCK(x) MUST_BE_READ_LOCKED(&ip_nat_lock)
 #define ASSERT_WRITE_LOCK(x) MUST_BE_WRITE_LOCKED(&ip_nat_lock)
@@ -268,8 +267,7 @@ void ip_nat_protocol_unregister(struct i
 	WRITE_UNLOCK(&ip_nat_lock);
 
 	/* Someone could be still looking at the proto in a bh. */
-	br_write_lock_bh(BR_NETPROTO_LOCK);
-	br_write_unlock_bh(BR_NETPROTO_LOCK);
+	synchronize_kernel();
 }
 
 static int init_or_cleanup(int init)
diff -puN net/ipv4/netfilter/ip_queue.c~brlock-6 net/ipv4/netfilter/ip_queue.c
--- 25/net/ipv4/netfilter/ip_queue.c~brlock-6	Tue Mar 11 16:59:51 2003
+++ 25-akpm/net/ipv4/netfilter/ip_queue.c	Tue Mar 11 16:59:51 2003
@@ -23,7 +23,6 @@
 #include <linux/netfilter_ipv4/ip_tables.h>
 #include <linux/netlink.h>
 #include <linux/spinlock.h>
-#include <linux/brlock.h>
 #include <linux/sysctl.h>
 #include <linux/proc_fs.h>
 #include <linux/security.h>
@@ -685,8 +684,7 @@ init_or_cleanup(int init)
 
 cleanup:
 	nf_unregister_queue_handler(PF_INET);
-	br_write_lock_bh(BR_NETPROTO_LOCK);
-	br_write_unlock_bh(BR_NETPROTO_LOCK);
+
 	ipq_flush(NF_DROP);
 	
 cleanup_sysctl:
diff -puN net/ipv4/protocol.c~brlock-6 net/ipv4/protocol.c
--- 25/net/ipv4/protocol.c~brlock-6	Tue Mar 11 16:59:51 2003
+++ 25-akpm/net/ipv4/protocol.c	Tue Mar 11 16:59:51 2003
@@ -37,7 +37,6 @@
 #include <linux/inet.h>
 #include <linux/netdevice.h>
 #include <linux/timer.h>
-#include <linux/brlock.h>
 #include <net/ip.h>
 #include <net/protocol.h>
 #include <net/tcp.h>
@@ -49,6 +48,7 @@
 #include <linux/igmp.h>
 
 struct inet_protocol *inet_protos[MAX_INET_PROTOS];
+static spinlock_t inet_proto_lock = SPIN_LOCK_UNLOCKED;
 
 /*
  *	Add a protocol handler to the hash tables
@@ -60,16 +60,14 @@ int inet_add_protocol(struct inet_protoc
 
 	hash = protocol & (MAX_INET_PROTOS - 1);
 
-	br_write_lock_bh(BR_NETPROTO_LOCK);
-
+	spin_lock_bh(&inet_proto_lock);
 	if (inet_protos[hash]) {
 		ret = -1;
 	} else {
 		inet_protos[hash] = prot;
 		ret = 0;
 	}
-
-	br_write_unlock_bh(BR_NETPROTO_LOCK);
+	spin_unlock_bh(&inet_proto_lock);
 
 	return ret;
 }
@@ -84,16 +82,15 @@ int inet_del_protocol(struct inet_protoc
 
 	hash = protocol & (MAX_INET_PROTOS - 1);
 
-	br_write_lock_bh(BR_NETPROTO_LOCK);
-
+	spin_lock_bh(&inet_proto_lock);
 	if (inet_protos[hash] == prot) {
 		inet_protos[hash] = NULL;
 		ret = 0;
 	} else {
 		ret = -1;
 	}
-
-	br_write_unlock_bh(BR_NETPROTO_LOCK);
+	spin_unlock_bh(&inet_proto_lock);
+	synchronize_kernel();
 
 	return ret;
 }

_
